#ifndef CBASEPERSISTANT_H
#define CBASEPERSISTANT_H

#include <iostream>
#include <vector>
#include <map>

namespace persistance {

// ________________________________________________________________________________________
/** \class CBasePersistant
 * Classe abstraite CBasePersistant : \n
 * CBasePersistant est une classe abstraite, base de la gestion de la persistance
 */
class CBasePersistant {

	public:
		/** Définition du pointeur sur une fonction constructeur.
		 * Chaque classe persistante devra définir une fonction constructeur sur ce modèle.
		 * Cette fonction lancera un constructeur de la classe chargé d'initialiser une occurrence d'après les données dans le flux. \n
		 * Chaque fonction constructeur devra être préalablement référencée. \n
		 * Exemples : CChainePersistante::constructeur - CChainePersistante::referencer
		 * \param prFlux : flux d'entrée
		 */
		typedef CBasePersistant* (*constructeur) (std::istream& prFlux);

		/** Constructeur CBasePersistant::CBasePersistant
		 * Initialise persistance et allocation. \n
		 * Les compteurs d'occurrences sont mis à jour. \n
		 * L'occurrence est ajoutée à la liste des objets persistants.
		 * \param pvPersistant précise si la classe doit être ou non persistante.
		 * \param pvAlloue précise si la classe est allouée. Permettra au destructeur de la classe de base de libérer la mémoire.
		 */
		CBasePersistant (bool pvPersistant, bool pvAlloue);

		/** Destructeur CBasePersistant::~CBasePersistant
		 * Détruit un objet persistant \n
		 * Les compteurs sont mis à jour. \n
		 * L'objet détruit est enlevé de la liste des objets persistants. \n
		 */
		virtual ~CBasePersistant ();

		/** Méthode CBasePersistant::taille : détermine la taille nécessaire pour enregistrer l'objet dans un flux. \n
		 * Cette taille sera variable selon la stratégie adoptée.
		 * \return taille de l'objet dans un flux.
		 */
		virtual uint64_t taille () const = 0;

		/** Méthode CBasePersistant::ecrire : enregistre l'objet dans un flux. \n
		 * Il s'agit d'une méthode virtuelle pure qui devra être obligatoirement redéfinie dans les classes dérivées.
		 * \param prFlux : Flux dans lequel écrire l'objet.
		 */
		virtual void ecrire (std::ostream& prFlux) const = 0;

	protected:
		/** Constructeur flux CBasePersistant::CBasePersistant : lecture d'un objet persistant depuis un flux. \n
		 * Constructeur protégé car il est appelé par la méthode statique CPersistant<TStrategie>::lire via le mécanisme de construction virtuelle. \n
		 * \param prFlux : Flux dans lequel écrire l'objet.
		 */
		CBasePersistant (std::istream& prFlux);

		/** Méthode statique de référencement d'une fonction de construction d'une classe via un flux. \n
		 * Crée le référentiel s'il n'existe pas déjà, \n
		 * Ajoute la référence (nom de classe, fonction de construction)
		 * \param kprNomClasse : Nom de la classe à référencer.
		 * \param fnConstructeur : Fonction de construction associée à un nom de classe.
		 */
		static void referencer (const std::string& kprNomClasse, constructeur fnConstructeur);

		/** Méthode CBasePersistant::toutEcrire : enregistre tous les objets déclarés persistants dans un flux. \n
		 * \param prFlux : Flux dans lequel écrire l'objet.
		 */
		static void toutEcrire (std::ostream& prFlux);

	private:
		CBasePersistant (const CBasePersistant& kprModele) = delete;
		CBasePersistant& operator= (const CBasePersistant& kprModele) = delete;

		bool mvPersistant; /**< Indicateur précisant si l'objet est ou non persistant */
		bool mvAlloue;/**< Indicateur précisant si l'objet est alloué sur la tas pour savoir comment libéré la mémoirej */

		static bool smvDestruction;/**< Indicateur de destruction pour éviter un débordement de pile d'appels */
		static uint64_t smvCompteur;/**< Compteur (variable de classe) d'objets */
		static uint64_t smvNbNonAlloue;/**< Compteur (variable de classe) d'objets alloués */
		static std::vector<CBasePersistant*>* smaLstObjets;/**< Liste des objets persistants */
		static std::map<std::string, CBasePersistant::constructeur>* smaReferentiel;/**< Référentiel associant nom de classe avec fonction de construction */
	};

};

#endif // CBASEPERSISTANT_H
