#ifndef CCHAINEPERSISTANTE_H
#define CCHAINEPERSISTANTE_H

#include "CPersistant.h"

namespace persistance {

// ________________________________________________________________________________________
/** \class CChainePersistante
 * Classe CChainePersistante basée sur une stratégie de stockage <TStrategie> pour sauvegarder des chaînes de caractères : \n
 * CChainePersistante<TStrategie> est une classe générique concrète, basée sur la classe générique de base CPersistant<TStrategie> \n
 * - La classe redéfinie les méthodes CChainePersistante::ecrire et CChainePersistante::nomclasse \n
 * - Elle définie un constructeur flux pour la lecture \n
 * - Elle définie une méthode statique CChainePersistante::constructeur pour référencer la classe \n
 * - Elle prépare une méthode CChainePersistante::referencer à appeler pour son référencement \n
 * Elle est basée sur une stratégie de stockage (exemple CBinaire).
 */
template <class TStrategie>
class CChainePersistante : public CPersistant<TStrategie> {

	public:
		/** Constructeur CChainePersistante::CChainePersistante \n
		 * Initialise la classe de base CPersistant et la chaîne persistante. \n
		 * \param kprChaine Valeur d'initialisation de la chaîne.
		 * \param pvPersistant précise si la classe doit être ou non persistante.
		 * \param pvAlloue précise si la classe est allouée. Permettra au destructeur de la classe de base de libérer la mémoire.
		 */
		CChainePersistante (const std::string& kprChaine, bool pvPersistant, bool pvAlloue);

		/** Destructeur CChainePersistante::~CChainePersistante \n
		 * Détruit une chaine persistante \n
		 */
		virtual ~CChainePersistante () {};

		/** Méthode CChainePersistante::ecrire : enregistre l'objet dans un flux. \n
		 * Commence par appeler l'écriture de la classe de base (obligatoire). \n
		 * Puis écrit la chaine selon la stratégie de stockage.
		 * \param prFlux : Flux dans lequel écrire l'objet.
		 */
		virtual void ecrire (std::ostream& prFlux) const;

		/** Méthode virtuelle CChainePersistante::nomClasse : Cette méthode renvoie le nom de la classe. \n
		 * Le nom de classe renvoyé intègre le schéma et la stratégie (exemple : "persistance::CChainePersistante<CBinaire>"). \n
		 * La stratégie doit donc pouvoir délivrer son propre nom.
		 * @return renvoie le nom de la classe.
		 */
		virtual std::string nomClasse () const { return smvNomClasse; };

		/** Méthode statique CChainePersistante::constructeur : Cette méthode construit par allocation un objet CChainePersistante. \n
		 * L'objet est créé et initialisé par appel au constructeur flux. \n
		 * @return renvoie un pointeur sur l'objet persistant alloué.
		 */
		static CBasePersistant* constructeur (std::istream& prFlux);

		/** Méthode statique CChainePersistante::referencer : Cette méthode référence la classe en associant le nom de classe avec sa méthode CChainePersistante::constructeur. \n
		 * Cette méthode doit être obligatoirement appelée par tout programme intéressé pour invoquer correctement la lecture. \PACDn
		 */
		static void referencer ();

	protected:

		/** Constructeur flux CChainePersistante::CChainePersistante \n
		 * Initialise l'objet à partir des données du flux. \n
		 * Il faut commencer par lire les données de la classe de base avec le constructeur flux hérité \n
		 * L'objet étant alloué et persistant, les attributs correspondants sont activés. \n
		 * \param prFlux Flux dans lequel lire la chaîne.
		 */
		CChainePersistante (std::istream& prFlux);

		CChainePersistante (const CChainePersistante& kprModele) = delete;
		CChainePersistante& operator= (const CChainePersistante& kprModele) = delete;

	private:
		std::string mvChaine; /**< Chaine persistante */

		static std::string smvNomClasse;
	};
};

#endif // CCHAINEPERSISTANTE_H
