#ifndef CPERSISTANT_H
#define CPERSISTANT_H

#include "CBasePersistant.h"

namespace persistance {

// ________________________________________________________________________________________
/** \class CPersistant
 * Classe abstraite CPersistant basée sur une stratégie de stockage : \n
 * CPersistant<TStrategie> est une classe abstraite, base de la gestion de la persistance selon une stratégie de stockage.
 */
// ________________________________________________________________________________________
template <class TStrategie>
class CPersistant : public CBasePersistant {

	public:
		/** Constructeur CPersistant::CPersistant
		 * Appelle le construteur de la classe de base CBasePersistant.
		 * \param pvPersistant précise si la classe doit être ou non persistante.
		 * \param pvAlloue précise si la classe est allouée. Permettra au destructeur de la classe de base de libérer la mémoire.
		 */
		CPersistant (bool pvPersistant, bool pvAlloue);

		/** Destructeur CPersistant::~CPersistant
		 * Détruit un objet persistant \n
		 * fait uniquement appel au destructeur de la classe de base CBasePersistant. \n
		 */
		virtual ~CPersistant () {};

	/** Méthode CPersistant::taille : détermine la taille nécessaire pour enregistrer l'objet dans un flux. \n
		 * Détermine la taille par enregistrement dans un flux mémoire.
		 * \return taille de l'objet dans un flux.
		 */
		virtual uint64_t taille () const;

		/** Méthode CPersistant::ecrire : enregistre l'objet dans un flux. \n
		 * Obligatoirement appelée par les classes dérivées pour enregistrer le nom de la classe dans le flux en fonction de la stratégie. \n
		 * La stratégie écrit le nom de la classe qui lui est transmise via CPersistant::nomClasse
		 * \param prFlux : Flux dans lequel écrire l'objet.
		 */
		virtual void ecrire (std::ostream& prFlux) const = 0;

		/** Méthode statique CPersistant::toutEcrire : enregistre un entête et tous les objets persistants dans un flux. \n
		 * Avant d'écrire les objets dans le flux, cette méthode demande à la Stratégie d'écrire un entête. \n
		 * L'écriture de l'ensemble des objets est confiée à la classe de base CBasePersistant::toutEcrire car c'est elle qui en gère la liste.
		 * \param prFlux : Flux dans lequel écrire l'objet.
		 */
		static void toutEcrire (std::ostream& prFlux);

		/** Méthode statique CPersistant::lire : Cette méthode est au cœur du processus de lecture. \n
		 * Elle commence par demander à la stratégie de lire un entête. \n
		 * Ensuite, d'objets en objets, elle commence - via la stratégie - par lire le nom de la classe. \n
		 * Elle recherche ensuite la fonction associée à cette classe puis l'invoque. \n
		 * Cette fonction associée devra alors initialiser une occurence (allouée) de cette classe via son constructeur flux qui se chargera de poursuivre la lecture.
		 * \param prFlux : Flux dans lequel lire l'objet.
		 */
		static void lire (std::istream& prFlux);

		/** Méthode virtuelle CPersistant::nomClasse : Cette méthode renvoie le nom de la classe. \n
		 * Elle devra obligatoirement être surchargée par toute classe concrète dérivée.
		 * @return renvoie le nom de la classe.
		 */
		virtual std::string nomClasse () const = 0;

	protected:
		/** Constructeur flux CPersistant::CPersistant
		 * Se contente de transmettre l'appel à la classe de base CBasePersistant::CBasePersistant.
		 * \param prFlux : Flux dans lequel lire la suite de l'objet.
		 */
		CPersistant (std::istream& prFlux);
	};

};

#endif // CPERSISTANT_H
